// Definicija klase za uskladistavanje podataka (skladiste.Skladiste).

package skladiste;
import java.util.*;

public class Skladiste<P> implements Iterable<P> {
  private static int posId = 0;  // Poslednji identifikator.
  private int id = ++posId;      // Identifikator skladita.
  private P[] niz;               // Niz za uskladitavanje proizvoda.
  private int ulaz=0, izlaz=0;   // Mesto stavljanja i uzimanja podatka.
  private int pun = 0;           // Broj popunjenih mesta.
  
  public Skladiste(int kap)                   // Inicijalizacija.
    { niz = (P[])new Object [kap]; }
                                              // Stavljanje proizvoda.
  public int id() { return id; }              // Dohvatanje identifikatora.

  public synchronized void stavi(P pro) throws InterruptedException {
    while (pun == niz.length) wait();
    niz[ulaz++] = pro;
    if (ulaz == niz.length) ulaz = 0;
    pun++;
    notifyAll();
  }
                                              // Uzimanje Proizvoda.
  public synchronized P uzmi() throws InterruptedException {
    while (pun == 0) wait();
    P pro = niz[izlaz]; niz[izlaz++] = null;
    if (izlaz == niz.length) izlaz = 0;
    pun--;
    notifyAll();
    return pro;
  }

  public Iterator<P> iterator()               // Stvaranje iteratora.
    { return new Iter(); }
  
  private class Iter implements Iterator<P> { // KLASA ITERATORA:
    
    private int sled = 0;                     // Sledei element.
    
    public boolean hasNext()                  // Ima li sledeeg? 
      { return sled < pun; }
    
    public P next() throws NoSuchElementException { // Dohvatanje sledeeg.
      if (sled == pun) throw new NoSuchElementException();
      P obj = niz[(izlaz+sled++) % niz.length];
      return obj;
    }
                                              // Izbacivanje nije podrano.
    public void remove() throws UnsupportedOperationException {
      throw new UnsupportedOperationException();
    }
  }
}
